@@ -984,9 +984,6 @@ event_page_fill_widgets (CompEditorPage *page, ECalComponent *comp)
 	cleanup_attendees (priv->deleted_attendees);
 	g_ptr_array_set_size (priv->deleted_attendees, 0);
 	
-	/* Component for cancellation */
-	priv->comp = e_cal_component_clone (comp);
-
 	/* Clean the page */
 	clear_widgets (epage);
 
@@ -2987,6 +2984,9 @@ init_widgets (EventPage *epage)
 	g_signal_connect((priv->categories), "changed",
 			    G_CALLBACK (field_changed_cb), epage);
 
+	/* emit signal when the group is changed */
+	g_signal_connect((priv->source_selector),"changed",G_CALLBACK(field_changed_cb),epage);
+
 	/* Set the default timezone, so the timezone entry may be hidden. */
 	zone = calendar_config_get_icaltimezone ();
 	e_timezone_entry_set_default_timezone (E_TIMEZONE_ENTRY (priv->start_timezone), zone);
