@@ -34,6 +34,7 @@
 #include <libgnomeui/gnome-dialog-util.h>
 #include <glade/glade.h>
 #include <e-util/e-dialog-widgets.h>
+#include <e-util/e-util-private.h>
 #include <misc/e-dateedit.h>
 #include "../calendar-config.h"
 #include "../e-meeting-time-sel.h"
@@ -78,7 +79,7 @@ static void schedule_page_set_dates (CompEditorPage *page, CompEditorPageDates *
 
 static void times_changed_cb (GtkWidget *widget, gpointer data);
 
-G_DEFINE_TYPE (SchedulePage, schedule_page, TYPE_COMP_EDITOR_PAGE);
+G_DEFINE_TYPE (SchedulePage, schedule_page, TYPE_COMP_EDITOR_PAGE)
 
 /* Class initialization function for the schedule page */
 static void
@@ -274,9 +275,6 @@ update_time (SchedulePage *spage, ECalComponentDateTime *start_date, ECalCompone
 static void
 clear_widgets (SchedulePage *spage)
 {
-	SchedulePagePrivate *priv;
-	
-	priv = spage->priv;
 }
 
 /* fill_widgets handler for the schedule page */
@@ -320,12 +318,6 @@ schedule_page_fill_widgets (CompEditorPage *page, ECalComponent *comp)
 static gboolean
 schedule_page_fill_component (CompEditorPage *page, ECalComponent *comp)
 {
-	SchedulePage *spage;
-	SchedulePagePrivate *priv;
-	
-	spage = SCHEDULE_PAGE (page);
-	priv = spage->priv;
-
 	return TRUE;
 }
 
@@ -408,11 +400,16 @@ SchedulePage *
 schedule_page_construct (SchedulePage *spage, EMeetingStore *ems)
 {
 	SchedulePagePrivate *priv;
+	char *gladefile;
 	
 	priv = spage->priv;
 
-	priv->xml = glade_xml_new (EVOLUTION_GLADEDIR 
-				   "/schedule-page.glade", NULL, NULL);
+	gladefile = g_build_filename (EVOLUTION_GLADEDIR,
+				      "schedule-page.glade",
+				      NULL);
+	priv->xml = glade_xml_new (gladefile, NULL, NULL);
+	g_free (gladefile);
+
 	if (!priv->xml) {
 		g_message ("schedule_page_construct(): "
 			   "Could not load the Glade XML file!");
@@ -473,6 +470,19 @@ schedule_page_new (EMeetingStore *ems)
 	return spage;
 }
 
+void
+schedule_page_set_name_selector (SchedulePage *spage, ENameSelector *name_selector)
+{
+	SchedulePagePrivate *priv;
+
+	g_return_if_fail (spage != NULL);
+	g_return_if_fail (IS_SCHEDULE_PAGE (spage));
+	
+	priv = spage->priv;
+
+	e_meeting_list_view_set_name_selector (priv->sel->list_view, name_selector);
+}
+
 static void
 times_changed_cb (GtkWidget *widget, gpointer data)
 {
