@@ -447,10 +447,10 @@ edit_memo_list_cb (EPopup *ep, EPopupItem *pitem, void *data)
 }
 
 static EPopupItem emc_source_popups[] = {
-	{ E_POPUP_ITEM, "10.new", N_("New Memo List"), new_memo_list_cb, NULL, "stock_notes", 0, 0 },
-	{ E_POPUP_ITEM, "15.copy", N_("Copy"), copy_memo_list_cb, NULL, "stock_folder-copy", 0, E_CAL_POPUP_SOURCE_PRIMARY },
-	{ E_POPUP_ITEM, "20.delete", N_("Delete"), delete_memo_list_cb, NULL, "stock_delete", 0, E_CAL_POPUP_SOURCE_USER|E_CAL_POPUP_SOURCE_PRIMARY },
-	{ E_POPUP_ITEM, "30.properties", N_("Properties..."), edit_memo_list_cb, NULL, "stock_folder-properties", 0, E_CAL_POPUP_SOURCE_PRIMARY },
+	{ E_POPUP_ITEM, "10.new", N_("_New Memo List"), new_memo_list_cb, NULL, "stock_notes", 0, 0 },
+	{ E_POPUP_ITEM, "15.copy", N_("_Copy"), copy_memo_list_cb, NULL, "stock_folder-copy", 0, E_CAL_POPUP_SOURCE_PRIMARY },
+	{ E_POPUP_ITEM, "20.delete", N_("_Delete"), delete_memo_list_cb, NULL, "stock_delete", 0, E_CAL_POPUP_SOURCE_USER|E_CAL_POPUP_SOURCE_PRIMARY },
+	{ E_POPUP_ITEM, "30.properties", N_("_Properties..."), edit_memo_list_cb, NULL, "stock_folder-properties", 0, E_CAL_POPUP_SOURCE_PRIMARY },
 };
 
 static void
@@ -1153,7 +1153,7 @@ impl__get_userCreatableItems (PortableServer_Servant servant,
 
 	list->_buffer[0].id = CREATE_MEMO_ID;
 	list->_buffer[0].description = _("New memo");
-	list->_buffer[0].menuDescription = _("_Memo");
+	list->_buffer[0].menuDescription = _("Mem_o");
 	list->_buffer[0].tooltip = _("Create a new memo");
 	list->_buffer[0].menuShortcut = 'o';
 	list->_buffer[0].iconName = "stock_insert-note";
@@ -1161,7 +1161,7 @@ impl__get_userCreatableItems (PortableServer_Servant servant,
 
 	list->_buffer[1].id = CREATE_MEMO_LIST_ID;
 	list->_buffer[1].description = _("New memo list");
-	list->_buffer[1].menuDescription = _("Memo l_ist");
+	list->_buffer[1].menuDescription = _("Memo li_st");
 	list->_buffer[1].tooltip = _("Create a new memo list");
 	list->_buffer[1].menuShortcut = '\0';
 	list->_buffer[1].iconName = "stock_notes";
