@@ -79,7 +79,7 @@ org_gnome_default_book (EPlugin *epl, EConfigHookItemFactoryData *data)
  
 	if (data->old)
 		return data->old;
-	widget = gtk_check_button_new_with_label (_("Mark as default folder"));
+	widget = gtk_check_button_new_with_mnemonic (_("Mark as _default folder"));
 	book_target = (EABConfigTargetSource *) data->target;
 	source = book_target->source;
     
@@ -94,6 +94,7 @@ org_gnome_default_book (EPlugin *epl, EConfigHookItemFactoryData *data)
 	return widget;
 }
 
+GtkWidget *org_gnome_default_cal (EPlugin *epl, EConfigHookItemFactoryData *data);
 
 GtkWidget *
 org_gnome_default_cal (EPlugin *epl, EConfigHookItemFactoryData *data)
@@ -105,7 +106,7 @@ org_gnome_default_cal (EPlugin *epl, EConfigHookItemFactoryData *data)
 
 	if (data->old)
 		return data->old;
-	widget = gtk_check_button_new_with_label (_("Mark as default folder"));
+	widget = gtk_check_button_new_with_mnemonic (_("Mark as _default folder"));
 	cal_target = (ECalConfigTargetSource *) data->target;
 	source = cal_target->source;
     
