@@ -261,7 +261,6 @@ cal_opened_cb (ECal *ecal, ECalendarStatus status, gpointer data)
 	FormatItipPObject *pitip = data;
 	ESource *source;
 	ECalSourceType source_type;
-	icaltimezone *zone;
 
 	source_type = e_cal_get_source_type (ecal);
 	source = e_cal_get_source (ecal);
@@ -274,6 +273,12 @@ cal_opened_cb (ECal *ecal, ECalendarStatus status, gpointer data)
 						      ITIP_VIEW_INFO_ITEM_TYPE_WARNING,
 						      _("Failed to load the calendar '%s'"),
 						      e_source_peek_name (source));
+		 
+		if (pitip->current_ecal == ecal) {
+			pitip->current_ecal = NULL;
+			itip_view_set_buttons_sensitive (ITIP_VIEW (pitip->view), FALSE);
+		}
+
 		g_hash_table_remove (pitip->ecals[source_type], e_source_peek_uid (source));
 		return;
 	}
@@ -287,8 +292,6 @@ cal_opened_cb (ECal *ecal, ECalendarStatus status, gpointer data)
 			itip_view_set_show_recur_check (ITIP_VIEW (pitip->view), FALSE);
 	}
 
-	zone = calendar_config_get_icaltimezone ();
-	e_cal_set_default_timezone (ecal, zone, NULL);
 
 	pitip->current_ecal = ecal;
 
@@ -299,6 +302,7 @@ static ECal *
 start_calendar_server (FormatItipPObject *pitip, ESource *source, ECalSourceType type, FormatItipOpenFunc func, gpointer data)
 {
 	ECal *ecal;
+	icaltimezone *zone = NULL;
 	
 	ecal = g_hash_table_lookup (pitip->ecals[type], e_source_peek_uid (source));
 	if (ecal) {
@@ -317,6 +321,9 @@ start_calendar_server (FormatItipPObject *pitip, ESource *source, ECalSourceType
 
 	g_hash_table_insert (pitip->ecals[type], g_strdup (e_source_peek_uid (source)), ecal);
 	
+	zone = calendar_config_get_icaltimezone ();
+	e_cal_set_default_timezone (ecal, zone, NULL);
+
 	e_cal_open_async (ecal, TRUE);
 
 	return ecal;
@@ -372,7 +379,6 @@ find_cal_opened_cb (ECal *ecal, ECalendarStatus status, gpointer data)
 	ESource *source;
 	ECalSourceType source_type;
 	icalcomponent *icalcomp;
-	icaltimezone *zone;
 	GList *objects = NULL;
 	
 	source_type = e_cal_get_source_type (ecal);
@@ -391,6 +397,11 @@ find_cal_opened_cb (ECal *ecal, ECalendarStatus status, gpointer data)
 						      ITIP_VIEW_INFO_ITEM_TYPE_WARNING,
 						      _("Failed to load the calendar '%s'"),
 						      e_source_peek_name (source));
+		if (pitip->current_ecal == ecal) {
+			pitip->current_ecal = NULL;
+			itip_view_set_buttons_sensitive (ITIP_VIEW (pitip->view), FALSE);
+		}
+			
 		g_hash_table_remove (pitip->ecals[source_type], e_source_peek_uid (source));
 		goto cleanup;
 	}
@@ -410,7 +421,7 @@ find_cal_opened_cb (ECal *ecal, ECalendarStatus status, gpointer data)
 	}
 
 
-	if (e_cal_get_object (ecal, fd->uid, fd->rid, &icalcomp, NULL)) {
+	if (!pitip->current_ecal && e_cal_get_object (ecal, fd->uid, fd->rid, &icalcomp, NULL)) {
 		icalcomponent_free (icalcomp);
 		
 		pitip->current_ecal = ecal;
@@ -435,8 +446,14 @@ find_cal_opened_cb (ECal *ecal, ECalendarStatus status, gpointer data)
 		set_buttons_sensitive (pitip);
 	}
 
-	zone = calendar_config_get_icaltimezone ();
-	e_cal_set_default_timezone (ecal, zone, NULL);
+	if (pitip->current_ecal && e_cal_get_static_capability (pitip->current_ecal, CAL_STATIC_CAPABILITY_RECURRENCES_NO_MASTER)) {
+		icalcomponent *icalcomp = e_cal_component_get_icalcomponent (pitip->comp);
+
+		if (check_is_instance (icalcomp))
+			itip_view_set_show_recur_check (ITIP_VIEW (pitip->view), TRUE);
+		else
+			itip_view_set_show_recur_check (ITIP_VIEW (pitip->view), FALSE);
+	}
 
  cleanup:
 	d(printf ("Decreasing itip formatter search count to %d\n", fd->count));
@@ -518,15 +535,20 @@ static void
 find_server (FormatItipPObject *pitip, ECalComponent *comp)
 {
 	FormatItipFindData *fd = NULL;
-	GSList *groups, *l;
+	GSList *groups, *l, *sources_conflict = NULL, *all_sources = NULL;
 	const char *uid;
 	const char *rid;
+	CamelFolder *folder;
+	CamelURL *url;
+	char *uri;
+	ESource *source = NULL, *current_source = NULL;
 
 	e_cal_component_get_uid (comp, &uid);
 	rid = e_cal_component_get_recurid_as_string (comp);
 
-	pitip->progress_info_id = itip_view_add_lower_info_item (ITIP_VIEW (pitip->view), ITIP_VIEW_INFO_ITEM_TYPE_PROGRESS, 
-								 _("Searching for an existing version of this appointment"));
+	folder = (((pitip->pobject).format)->format).folder;
+        url = CAMEL_SERVICE (folder->parent_store)->url;
+        uri = camel_url_to_string (url, CAMEL_URL_HIDE_ALL);
 
 	itip_view_set_buttons_sensitive (ITIP_VIEW (pitip->view), FALSE);
 
@@ -539,36 +561,76 @@ find_server (FormatItipPObject *pitip, ECalComponent *comp)
 
 		sources = e_source_group_peek_sources (group);
 		for (m = sources; m; m = m->next) {
-			ESource *source;
-			ECal *ecal;
+                        char *source_uri = NULL;
 			
 			source = m->data;
 			
-			if (!fd) {
-				char *start = NULL, *end = NULL;
-				
-				fd = g_new0 (FormatItipFindData, 1);
-				fd->pitip = pitip;
-				fd->uid = g_strdup (uid);
-				fd->rid = g_strdup (rid);
-				
-				if (pitip->start_time && pitip->end_time) {
-					start = isodate_from_time_t (pitip->start_time);
-					end = isodate_from_time_t (pitip->end_time);
-					
-					fd->sexp = g_strdup_printf ("(and (occur-in-time-range? (make-time \"%s\") (make-time \"%s\")) (not (uid? \"%s\")))", 
-								    start, end, icalcomponent_get_uid (pitip->ical_comp));
-				}
-				
-				g_free (start);
-				g_free (end);
-			}
-			fd->count++;
-			d(printf ("Increasing itip formatter search count to %d\n", fd->count));
+			if (e_source_get_property (source, "conflict"))
+				sources_conflict = g_slist_prepend (sources_conflict, source);
 
-			ecal = start_calendar_server (pitip, source, pitip->type, find_cal_opened_cb, fd);
-		}		
+			if (current_source)
+				continue;
+			
+			source_uri = e_source_get_uri (source);
+			if (source_uri && (strcmp (uri, source_uri) == 0)) {
+				current_source = source;
+				sources_conflict = g_slist_prepend (sources_conflict, source);
+
+				g_free (source_uri);
+				continue;
+			}
+	
+			all_sources = g_slist_prepend (all_sources, source);
+			g_free (source_uri);
+	
+		}
 	}
+	
+	if (current_source)		
+		l = sources_conflict;
+	else {
+		pitip->progress_info_id = itip_view_add_lower_info_item (ITIP_VIEW (pitip->view), ITIP_VIEW_INFO_ITEM_TYPE_PROGRESS, 
+				_("Searching for an existing version of this appointment"));
+
+		l = all_sources;
+	}
+
+	for (; l != NULL; l = l->next) {
+		ECal *ecal;
+		source = l->data;
+
+		if (!fd) {
+			char *start = NULL, *end = NULL;
+
+			fd = g_new0 (FormatItipFindData, 1);
+			fd->pitip = pitip;
+			fd->uid = g_strdup (uid);
+			fd->rid = g_strdup (rid);
+
+			if (pitip->start_time && pitip->end_time) {
+				start = isodate_from_time_t (pitip->start_time);
+				end = isodate_from_time_t (pitip->end_time);
+
+				fd->sexp = g_strdup_printf ("(and (occur-in-time-range? (make-time \"%s\") (make-time \"%s\")) (not (uid? \"%s\")))", 
+						start, end, icalcomponent_get_uid (pitip->ical_comp));
+			}
+
+			g_free (start);
+			g_free (end);
+		}
+		fd->count++;
+		d(printf ("Increasing itip formatter search count to %d\n", fd->count));
+
+		if (current_source == source)
+			pitip->current_ecal = start_calendar_server (pitip, source, pitip->type, find_cal_opened_cb, fd);
+		else 
+			ecal = start_calendar_server (pitip, source, pitip->type, find_cal_opened_cb, fd);
+
+	}		
+
+	g_slist_free (all_sources);
+	g_slist_free (sources_conflict);
+	g_free (uri);
 }
 
 static void
@@ -734,7 +796,6 @@ update_item (FormatItipPObject *pitip, ItipViewResponse response)
 		CamelMimeMessage *msg = ((EMFormat *) pitip->pobject.format)->message;
 		
 		e_cal_component_get_attachment_list (clone_comp, &attachments);
-		g_message ("Number of attachments is %d", g_slist_length (attachments));
 		
 		for (l = attachments; l; l = l->next) {
 			GSList *parts = NULL, *m;
@@ -755,7 +816,6 @@ update_item (FormatItipPObject *pitip, ItipViewResponse response)
 						continue;
 
 					new_uri = em_utils_temp_save_part (NULL, part);
-					g_message ("DEBUG: the uri obtained was %s\n", new_uri);
 					new_attachments = g_slist_append (new_attachments, new_uri);
 				}
 
@@ -773,6 +833,9 @@ update_item (FormatItipPObject *pitip, ItipViewResponse response)
 				new_attachments = g_slist_append (new_attachments, g_strdup (uri));
 			}
 		}
+
+		g_slist_foreach (attachments, g_free, NULL);
+		g_slist_free (attachments);
 				
 		e_cal_component_set_attachment_list (clone_comp, new_attachments);
 	}
@@ -1345,7 +1408,7 @@ static void
 view_response_cb (GtkWidget *widget, ItipViewResponse response, gpointer data)
 {
 	FormatItipPObject *pitip = data;
-	gboolean status = FALSE;
+	gboolean status = FALSE, delete_invitation_from_cache = FALSE;
 	icalproperty *prop;
 	ECalComponentTransparency trans;
 
@@ -1367,53 +1430,67 @@ view_response_cb (GtkWidget *widget, ItipViewResponse response, gpointer data)
 	}
 
 	switch (response) {
-	case ITIP_VIEW_RESPONSE_ACCEPT:
-		status = change_status (pitip->ical_comp, pitip->my_address, 
+		case ITIP_VIEW_RESPONSE_ACCEPT:
+			status = change_status (pitip->ical_comp, pitip->my_address, 
 					ICAL_PARTSTAT_ACCEPTED);
-		if (status) {
-			e_cal_component_rescan (pitip->comp);
-			update_item (pitip, response);
-		}
-		break;
-	case ITIP_VIEW_RESPONSE_TENTATIVE:
-		status = change_status (pitip->ical_comp, pitip->my_address,
+			if (status) {
+				e_cal_component_rescan (pitip->comp);
+				update_item (pitip, response);
+				if (e_cal_get_save_schedules (pitip->current_ecal))
+					delete_invitation_from_cache = TRUE;
+			}
+			break;
+		case ITIP_VIEW_RESPONSE_TENTATIVE:
+			status = change_status (pitip->ical_comp, pitip->my_address,
 					ICAL_PARTSTAT_TENTATIVE);
-		if (status) {
-			e_cal_component_rescan (pitip->comp);
-			update_item (pitip, response);
-		}
-		break;
-	case ITIP_VIEW_RESPONSE_DECLINE:
-		status = change_status (pitip->ical_comp, pitip->my_address,
+			if (status) {
+				e_cal_component_rescan (pitip->comp);
+				update_item (pitip, response);
+				if (e_cal_get_save_schedules (pitip->current_ecal))
+					delete_invitation_from_cache = TRUE;
+
+			}
+			break;
+		case ITIP_VIEW_RESPONSE_DECLINE:
+			status = change_status (pitip->ical_comp, pitip->my_address,
 					ICAL_PARTSTAT_DECLINED);
-		if (status) {
-			e_cal_component_rescan (pitip->comp);
+			if (status) {
+				e_cal_component_rescan (pitip->comp);
+				update_item (pitip, response);
+				if (e_cal_get_save_schedules (pitip->current_ecal))
+					delete_invitation_from_cache = TRUE;
+			}
+			break;
+		case ITIP_VIEW_RESPONSE_UPDATE:
+			update_attendee_status (pitip);
+			break;
+		case ITIP_VIEW_RESPONSE_CANCEL:
 			update_item (pitip, response);
-		}
-		break;
-	case ITIP_VIEW_RESPONSE_UPDATE:
-		update_attendee_status (pitip);
-		break;
-	case ITIP_VIEW_RESPONSE_CANCEL:
-		update_item (pitip, response);
-		break;
-	case ITIP_VIEW_RESPONSE_REFRESH:
-		send_item (pitip);
-		break;
-	case ITIP_VIEW_RESPONSE_OPEN:
-		g_idle_add (idle_open_cb, pitip);
-		return;
-	default:
-		break;
+			break;
+		case ITIP_VIEW_RESPONSE_REFRESH:
+			send_item (pitip);
+			break;
+		case ITIP_VIEW_RESPONSE_OPEN:
+			g_idle_add (idle_open_cb, pitip);
+			return;
+		default:
+			break;
 	}
 
-	if (pitip->delete_message) {
+	if (delete_invitation_from_cache) {
+		CamelFolderChangeInfo *changes = NULL;
+		camel_folder_summary_remove_uid(((EMFormat *) pitip->pobject.format)->folder->summary, ((EMFormat *) pitip->pobject.format)->uid);
+		changes = camel_folder_change_info_new ();
+		camel_folder_change_info_remove_uid (changes, (char *)((EMFormat *) pitip->pobject.format)->uid);
+		camel_object_trigger_event (((EMFormat *) pitip->pobject.format)->folder, "folder_changed", changes);
+		camel_folder_change_info_free (changes);
+	}
+
+	if (!delete_invitation_from_cache && pitip->delete_message) {
 		g_message ("Deleting!");
 		camel_folder_delete_message (((EMFormat *) pitip->pobject.format)->folder, ((EMFormat *) pitip->pobject.format)->uid);
 	}
 	
-        if (e_cal_get_save_schedules (pitip->current_ecal))
-                return;
 
         if (itip_view_get_rsvp (ITIP_VIEW (pitip->view)) && status) {
                 ECalComponent *comp = NULL;
@@ -1571,9 +1648,15 @@ format_itip_object (EMFormatHTML *efh, GtkHTMLEmbedded *eb, EMFormatHTMLPObject
 	case ICAL_METHOD_DECLINECOUNTER:
 		itip_view_set_mode (ITIP_VIEW (pitip->view), ITIP_VIEW_MODE_DECLINECOUNTER);
 		break;
-	default:
-		g_assert_not_reached ();
+	case ICAL_METHOD_X :
+		/* Handle appointment requests from Microsoft Live. This is
+		 * a best-at-hand-now handling. Must be revisited when we have
+		 * better access to the source of such meetings */
+		pitip->method = ICAL_METHOD_REQUEST;
+		itip_view_set_mode (ITIP_VIEW (pitip->view), ITIP_VIEW_MODE_REQUEST);
 		break;
+	default:
+		return FALSE;
 	}
 
 	itip_view_set_item_type (ITIP_VIEW (pitip->view), pitip->type);
@@ -1769,48 +1852,8 @@ format_itip_object (EMFormatHTML *efh, GtkHTMLEmbedded *eb, EMFormatHTMLPObject
 	if (pitip->calendar_uid)
 		pitip->current_ecal = start_calendar_server_by_uid (pitip, pitip->calendar_uid, pitip->type);
 	else {
-
-		/* Since the mailer uri matches with only groupwise calendar uri so for this case we need not 
-		   have to call find_server */
-                CamelFolder *folder;
-                CamelURL *url;
-                char *uri;
-		/* *l is already declared in this function */
-                GSList *groups, *l;
-                ESource *source = NULL;
-                gboolean found = FALSE;
-
-                folder = (((pitip->pobject).format)->format).folder;
-                url = CAMEL_SERVICE (folder->parent_store)->url;
-                uri = camel_url_to_string (url, CAMEL_URL_HIDE_ALL);
-
-                groups = e_source_list_peek_groups (pitip->source_lists[pitip->type]);
-                for (l = groups; l && !found; l = l->next) {
-                        ESourceGroup *group;
-                        GSList *sources, *m;
-
-                        group = E_SOURCE_GROUP (l->data);
-                        sources = e_source_group_peek_sources (group);
-                        for (m = sources; m && !found; m = m->next) {
-                               char *source_uri = NULL;
-
-                               source = E_SOURCE (m->data);
-                               source_uri = e_source_get_uri (source);
-                               
-			       if (source_uri)
-				       found = (strcmp (uri, source_uri) == 0);
-                               
-			       g_free (source_uri);                            
-                        }
-                }
-
-                if (found) {
-                        pitip->current_ecal = start_calendar_server (pitip, source, pitip->type, cal_opened_cb, pitip);
-                        set_buttons_sensitive (pitip);
-                } else
-                        find_server (pitip, pitip->comp);
-	
-		g_free (uri);
+		find_server (pitip, pitip->comp);
+		set_buttons_sensitive (pitip);
 	}
 	
 	return TRUE;
