@@ -139,8 +139,7 @@ ask_destination_and_save (EPlugin *ep, ECalPopupTargetSource *target, ECalSource
 			renderer, "text", DEST_NAME_COLUMN, NULL);
 
 	while (format_handlers) {
-		FormatHandler *handler = format_handlers->data;
-
+		handler = format_handlers->data;
 		gtk_list_store_append (store, &iter);
 		gtk_list_store_set (store, &iter, DEST_NAME_COLUMN, 
 			handler->combo_label, -1);
@@ -176,6 +175,7 @@ ask_destination_and_save (EPlugin *ep, ECalPopupTargetSource *target, ECalSource
 
 	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
 	gtk_file_chooser_set_extra_widget (GTK_FILE_CHOOSER (dialog), extra_widget);
+	gtk_file_chooser_set_local_only (dialog, FALSE);
 #else
 	dialog = gtk_file_selection_new (_("Select destination file"));
 	gtk_box_pack_start (GTK_BOX (GTK_FILE_SELECTION (dialog)->main_vbox), extra_widget, FALSE, TRUE, 0);
