@@ -177,6 +177,8 @@ do_adjustment (gpointer user_data)
 	adj = gtk_layout_get_hadjustment (GTK_LAYOUT (GNOME_CANVAS_ITEM (reflow)->canvas));
 	value = adj->value;
 
+	if ((!reflow->items) || (!reflow->items[row]))
+		return TRUE;
 	min_value = reflow->items[row]->x2 - adj->page_size;
 	max_value = reflow->items[row]->x1;
 
@@ -830,13 +832,11 @@ static void
 e_reflow_realize (GnomeCanvasItem *item)
 {
 	EReflow *reflow;
-	GnomeCanvasGroup *group;
 	GtkAdjustment *adjustment;
 	int count;
 	int i;
 
 	reflow = E_REFLOW (item);
-	group = GNOME_CANVAS_GROUP (item);
 
 	if (GNOME_CANVAS_ITEM_CLASS(parent_class)->realize)
 		(* GNOME_CANVAS_ITEM_CLASS(parent_class)->realize) (item);
@@ -951,7 +951,6 @@ e_reflow_event (GnomeCanvasItem *item, GdkEvent *event)
 				}
 			}
 #endif
-			break;
 		case GDK_BUTTON_PRESS:
 			switch(event->button.button) 
 				{
